(*
  File: dft.ml
  Author: Shivkumar Chandrasekaran
  Date: 10.24.2006
  Purpose: Discrete Fourier Transform

  $Id$
 *)


open Nla

module Dfter (CBlop : BLOP with type t = Complex.t) =
  struct

    open CBlop

    let pi = 4.0 *. atan 1.0

    let ( +: ) x y = {Complex.re = x; Complex.im = y}
    let ( *% ) z w = Complex.mul z w

    let dft n =
      let omega = 2.0 *. pi /. float n in
      fun2mat (fun i j -> Complex.exp (0.0 +: omega *. float ((i-1) * (j-1))))
	n n

    let semisep x y =
      let m = noOfRows x in
      assert (m = noOfRows y);
      assert (1 = noOfCols x && 1 = noOfCols y);
      fun2mat (fun i j ->
	if i <= j then
	  (x $@ (i,1)) *% (y $@ (j, 1))
	else
	  (y $@ (i,1)) *% (x $@ (j, 1)))
	m m

    let exp n k =
      let omega = 2.0 *. pi /. float n in
      fun2mat (fun i j -> assert (j = 1);
	Complex.exp (0.0 +: omega *. float k))
	n 1

    let step n k =
      fun2mat (fun i j -> assert (j = 1);
	if i < k then Complex.zero else Complex.one)
	n 1

    let delta n k =
      fun2mat (fun i j -> assert (j = 1);
	if i = k then Complex.one else Complex.zero)
	n 1

    let pow n k =
      fun2mat (fun i j -> assert (j = 1);
	(float i ** float k +: 0.0))
	n 1

    let dft2D a =
      let m = noOfRows a
      and n = noOfCols a in
      let r = dft n in
      conjg' r;
      (dft m) *$ (a *$~ r)

    let dft2S a =
      let m = noOfRows a
      and n = noOfCols a in
      (dft m) *$ (a *$~ (dft n))

    let diag x =
      let m = noOfRows x in
      assert (noOfCols x = 1);
      fun2mat (fun i j -> if i = j then x $@ (i, 1) else Complex.zero)
	m m

    let cShift n =
      fun2mat (fun i j ->
	if i = j + 1 || (i = 1 && j = n) then Complex.one
	else Complex.zero)
	n n

  end (* module Dfter (Blop : BLOP) *)


module ZDft = Dfter (ZBlop)
module CDft = Dfter (CBlop)

