(*
   File: utils.ml
   Author: Shivkumar Chandrasekaran
   Date: 9.11.2003
   Email: shiv@ece.ucsb.edu
   Package: CamlFloat
   Purpose: Common miscellaneous types and routines

   $Id$
*)

type ('a, 'b) btree =
    Leaf of 'a
  | Node of 'b * ('a, 'b) btree * ('a, 'b) btree

let rec structurallyEqual a b =
  match a, b with
  | Leaf _, Leaf _ -> true
  | Node _, Leaf _ -> false
  | Leaf _, Node _ -> false
  | Node (_, al, ar), Node (_, bl, br) ->
      structurallyEqual al bl && structurallyEqual ar br

let getNodeData tree =
  match tree with
  | Leaf d -> d
  | Node (d, _, _) -> d

let ppBtree ppLeaf ppNode tree =
  let rec pp tree =
    match tree with
    | Leaf d -> 
	Format.open_box 0;
	Format.print_string "Leaf"; Format.print_space ();
	ppLeaf d;
	Format.close_box ()
    | Node (d, lTree, rTree) ->
	Format.open_box 0;
	Format.print_string "Node"; Format.print_space ();
	ppNode d; Format.print_space ();
	Format.open_vbox 0;
	pp lTree;
	Format.print_space ();
	pp rTree;
	Format.close_box ();
	Format.close_box ()
  in
  Format.open_vbox 0;
  Format.print_space ();
  pp tree;
  Format.print_space ();
  Format.close_box ()

let ppList pElem aList =
  let prev = ref false in
  let rec pp aList =
    match aList with
    | [] -> Format.print_char ']'
    | a :: rs ->
	if !prev then begin
	  Format.print_char ';'; Format.print_space ();
	end else
	  prev := true;
	pElem a;
	pp rs
  in
  Format.open_box 0;
  Format.print_char '[';
  pp aList;
  Format.close_box ()

let pp2Tuple p1 p2 tuple =
  Format.open_box 0;
  Format.print_char '(';
  p1 (fst tuple);
  Format.print_char ',';
  Format.print_space ();
  p2 (snd tuple);
  Format.print_char ')';
  Format.close_box ()

let ppArray ppElem xs =
  let n = Array.length xs in
  Format.open_box 0;
  Format.print_string "[|";
  if n > 0 then
    ppElem xs.(0);
  for i = 1 to n - 1 do
    Format.print_char ';';
    Format.print_space ();
    ppElem xs.(i);
  done;
  Format.print_string "|]";
  Format.close_box ()

let ppFloat x =
  Format.open_box 0;
  Format.print_float x;
  Format.close_box ()

