(*
   File: testNla.ml
   Author: Shivkumar Chandrasekaran
   Email: shiv@ece.ucsb.edu
   Date: 8.29.2003
   Package: CamlFloat
   Purpose: Tests for Nla

   $Id: testNla.ml,v 1.2 2004/06/04 21:50:34 shiv Exp shiv $
*)

open Nla;;


module Tester (Blop : BLOP) =
struct

  open Blop

  let test1 () =
    let ms = [| 10; 11; 12; 13; 14; 15 |]
    and ns = [| 10; 11; 12; 13; 14; 15 |]
    and ks = [| 1; 2; 3; 4; 5; 6 |]
    and ok = ref true in
    for m = 0 to Array.length ms - 1 do
      for n = 0 to Array.length ns - 1 do
	for k = 0 to Array.length ks - 1 do
	  for ooc = 0 to 1 do
	    let a = randomMatrix ~ooc:(ooc=1) ms.(m) ns.(n)
	    and x = randomMatrix ~ooc:(ooc=1) ns.(n) ks.(k) in
	    let b = a *$ x in
	    let xs = a /$ b in
	    ok := !ok &&
	      ((norm1 (a *$ xs -$ b) /.
		(norm1 a *. norm1 xs +. norm1 b)) <=
	       sqrt (float (ms.(m) * ns.(n) * ks.(k))) *. machine_epsilon)
	  done
	done
      done
    done;
    !ok

  let test2 () =
    let ms = [| 20; 21; 22; 23; 24; 25 |]
    and ns = [| 20; 21; 22; 23; 24; 25 |]
    and ok = ref true in
    for m = 0 to Array.length ms - 1 do
      for n = 0 to Array.length ns - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(m)
	  and n = ns.(n) in
	  let a = randomMatrix ~ooc:(ooc=1) m n in
	  let (q, lF) = ql' (copy ~ooc:(ooc=1) a)
	  and l = zeros ~ooc:(ooc=1) m n in
	  if m > n then
	    iDL lF (l $ (1 + m - n <-> m, All))
	  else
	    iDL lF l;
	  q @* l;
	  ok := !ok && (norm1 (l -$ a) <= sqrt (float (m * n)) *.
			  machine_epsilon *. norm1 a)
	done
      done
    done;
    !ok

  let test3 () =
    let ms = [| 20; 21; 22; 23; 24; 25 |]
    and ns = [| 20; 21; 22; 23; 24; 25 |]
    and ok = ref true in
    for m = 0 to Array.length ms - 1 do
      for n = 0 to Array.length ns - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(m)
	  and n = ns.(n) in
	  let a = randomMatrix ~ooc:(ooc=1) m n in
	  let (lF, q) = lq' (copy ~ooc:(ooc=1) a)
	  and l = zeros ~ooc:(ooc=1) m n in
	  if n > m then
	    iDL lF (l $ (All, 1 <-> m))
	  else
	    iDL lF l;
	  l *@ q;
	  ok := !ok && (norm1 (l -$ a) <= sqrt (float (m * n)) *.
			  machine_epsilon *. norm1 a)
	done
      done
    done;
    !ok

  let test4 () =
    let ms = [| 20; 21; 22; 23; 24; 25 |]
    and ns = [| 20; 21; 22; 23; 24; 25 |]
    and ok = ref true in
    for m = 0 to Array.length ms - 1 do
      for n = 0 to Array.length ns - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(m)
	  and n = ns.(n) in
	  let a = randomMatrix ~ooc:(ooc=1) m n in
	  let (q, lF) = ql' (copy ~ooc:(ooc=1) a)
	  and l = zeros ~ooc:(ooc=1) m n in
	  if m > n then
	    iDL lF (l $ (1 + m - n <-> m, All))
	  else
	    iDL lF l;
	  q @~* a;
	  ok := !ok && (norm1 (l -$ a) <= sqrt (float (m * n)) *.
			  machine_epsilon *. norm1 a)
	done
      done
    done;
    !ok

  let test5 () =
    let ms = [| 20; 21; 22; 23; 24; 25 |]
    and ns = [| 20; 21; 22; 23; 24; 25 |]
    and ok = ref true in
    for mi = 0 to Array.length ms - 1 do
      for ni = 0 to Array.length ns - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(mi)
	  and n = ns.(ni) in
	  let a = randomMatrix ~ooc:(ooc=1) m n in
	  let (lF, q) = lq' (copy ~ooc:(ooc=1) a)
	  and l = zeros ~ooc:(ooc=1) m n in
	  if n > m then
	    iDL lF (l $ (All, 1 <-> m))
	  else
	    iDL lF l;
	  a *~@ q;
	  ok := !ok && (norm1 (l -$ a) <= sqrt (float (m * n)) *.
			  machine_epsilon *. norm1 a)
	done
      done
    done;
    !ok

  let test6 () =
    let ms = [| 10; 11; 12; 13; 14; 15; 16 |]
    and ns = [| 10; 11; 12; 13; 14; 15; 16 |]
    and ok = ref true in
    for mi = 0 to Array.length ms - 1 do
      for ni = 0 to Array.length ns - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(mi)
	  and n = ns.(ni) in
	  let a = randomMatrix ~ooc:(ooc=1) m n in
	  let (u, s, vt) = svd' ~ooc:(ooc=1) (copy ~ooc:(ooc=1) a) in
	  rowScale_real' s vt;
	  ok := !ok && (norm1 (u *$ vt -$ a) <= machine_epsilon *. norm1
			  a *. (float (m * n)))
	done
      done
    done;
    !ok

  let test7 () =
    let ms = [| 10; 11; 12; 13; 14; 15; 16 |]
    and ns = [| 10; 11; 12; 13; 14; 15; 16 |]
    and ok = ref true in
    for mi = 0 to Array.length ms - 1 do
      for ni = 0 to Array.length ns - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(mi)
	  and n = ns.(ni) in
	  let a = randomMatrix ~ooc:(ooc=1) m n in
	  let (u, s, vt) = svd' ~ooc:(ooc=1) (copy ~ooc:(ooc=1) a) in
	  colScale_real' u s;
	  ok := !ok && (norm1 (u *$ vt -$ a) <= machine_epsilon *. norm1
			  a *. (float (m * n)))
	done
      done
    done;
    !ok

  let test8 () =
    let ms = [| 20; 21; 22; 23; 24; 25 |]
    and ns = [| 20; 21; 22; 23; 24; 25 |]
    and nrhss = [| 1; 2; 3; 4 |]
    and ok = ref true in
    for mi = 0 to Array.length ms - 1 do
      for ni = 0 to Array.length ns - 1 do
	for nrhsi = 0 to Array.length nrhss - 1 do
	  for ooc = 0 to 1 do
	    let m = ms.(mi)
	    and n = ns.(ni)
	    and nrhs = nrhss.(nrhsi) in
	    let a = randomMatrix ~ooc:(ooc=1) m n
	    and x = randomMatrix ~ooc:(ooc=1) n nrhs in
	    let b = a *$ x in
	    if m >= n then
	      let (q, l) = ql' (copy ~ooc:(ooc=1) a)
	      and bc = copy ~ooc:(ooc=1) b in
	      q @~* bc;
	      subs' l (bc $ (m - n + 1 <-> m, All));
	      let xh = bc $ (m - n + 1 <-> m, All) in
	      ok := !ok && (norm1 (a *$ xh -$ b) <= machine_epsilon
			      *. sqrt (float (max m n))
			      *. (norm1 a *. norm1 xh +. norm1 b))
	    else
	      let (l, q) = lq' (copy ~ooc:(ooc=1) a)
	      and bc = zeros ~ooc:(ooc=1) n nrhs in
	      (bc $ (1 <-> m, All)) <-$ b;
	      subs' l (bc $ (1 <-> m, All));
	      q @~* bc;
	      ok := !ok && (norm1 (a *$ bc -$ b) <= machine_epsilon
			      *. sqrt (float (max m n))
			      *. (norm1 a *. norm1 bc +. norm1 b))
	  done
	done
      done
    done;
    !ok

  let test9 () =
    let ms = [| 10; 20; 30; 40 |]
    and nrhss = [| 1; 2; 3; 4 |]
    and ok = ref true in
    for mi = 0 to Array.length ms - 1 do
      for nrhsi = 0 to Array.length nrhss - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(mi)
	  and nrhs = nrhss.(nrhsi) in
	  let a = randomMatrix ~ooc:(ooc=1) m m
	  and b = randomMatrix ~ooc:(ooc=1) m nrhs in
	  let x = copy ~ooc:(ooc=1) b in
	  pluSolve' (copy ~ooc:(ooc=1) a) x;
	  ok := !ok && (norm1 (a *$ x -$ b) <= 2.0 *. machine_epsilon *.
			  (norm1 a *. norm1 x +. norm1 b))
	done
      done;
    done;
    !ok

  let test10 () =
    let ms = [| 10; 20; 30; 40 |]
    and ok = ref true in
    for mi = 0 to Array.length ms - 1 do
      for ooc = 0 to 1 do
	let m = ms.(mi) in
	let a = randomMatrix ~ooc:(ooc=1) m m in
	let (p, l, u) = plu' (copy ~ooc:(ooc=1) a) in
	let lc = copyL ~ooc:(ooc=1) l
	and uc = copyU ~ooc:(ooc=1) u in
	p @* lc;
	ok := !ok && norm1 (lc *$ uc -$ a) <= machine_epsilon *.
	    (norm1 a +. norm1 lc *. norm1 uc)
      done
    done;
    !ok

  let randomMmapdMatrix m n =
    let genRandFloat _ _ = randn ()
    and a = mmapdMatrix ~shared:false (Filename.temp_file "rmm" ".dat") m n in
    fun2mat' genRandFloat a;
    a

  let test11 () =
    let ms = [| 10; 20; 30; 40 |]
    and nrhss = [| 1; 2; 3; 4 |]
    and ok = ref true in
    for mi = 0 to Array.length ms - 1 do
      for nrhsi = 0 to Array.length nrhss - 1 do
	let m = ms.(mi)
	and nrhs = nrhss.(nrhsi) in
	let a = randomMmapdMatrix m m
	and b = randomMmapdMatrix m nrhs in
	let x = copy b in
	pluSolve' (copy a) x;
	ok := !ok && (norm1 (a *$ x -$ b) <= 2.0 *. machine_epsilon *.
			(norm1 a *. norm1 x +. norm1 b))
      done;
    done;
    !ok

  let test12 () =
    let ms = [| 22; 23; 24 |]
    and ns = [| 22; 23; 24 |]
    and ok = ref true in
    for m = 0 to Array.length ms - 1 do
      for n = 0 to Array.length ns - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(m)
	  and n = ns.(n) in
	  let a = randomMatrix ~ooc:(ooc=1) m n in
	  let (q, rF) = qr' (copy ~ooc:(ooc=1) a)
	  and r = zeros ~ooc:(ooc=1) m n in
	  if m > n then
	    iDU rF (r $ (1 <-> n, All))
	  else
	    iDU rF r;
	  q @* r;
	  ok := !ok && (norm1 (r -$ a) <= sqrt (float (m * n)) *.
			  machine_epsilon *. norm1 a)
	done
      done
    done;
    !ok

  let test13 () =
    let ms = [| 20; 21; 22; 23; 24; 25 |]
    and ns = [| 20; 21; 22; 23; 24; 25 |]
    and ok = ref true in
    for m = 0 to Array.length ms - 1 do
      for n = 0 to Array.length ns - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(m)
	  and n = ns.(n) in
	  let a = randomMatrix ~ooc:(ooc=1) m n in
	  let (q, rF) = qr' (copy ~ooc:(ooc=1) a)
	  and r = zeros ~ooc:(ooc=1) m n in
	  if m > n then
	    iDU rF (r $ (1 <-> n, All))
	  else
	    iDU rF r;
	  q @~* a;
	  ok := !ok && (norm1 (r -$ a) <= sqrt (float (m * n)) *.
			  machine_epsilon *. norm1 a)
	done
      done
    done;
    !ok

  let test14 () =
    let ms = [| 20; 21; 22; 23; 24; 25 |]
    and ns = [| 20; 21; 22; 23; 24; 25 |]
    and nrhss = [| 1; 2; 3; 4 |]
    and ok = ref true in
    for mi = 0 to Array.length ms - 1 do
      for ni = 0 to Array.length ns - 1 do
	for nrhsi = 0 to Array.length nrhss - 1 do
	  for ooc = 0 to 1 do
	    let m = ms.(mi)
	    and n = ns.(ni)
	    and nrhs = nrhss.(nrhsi) in
	    let a = randomMatrix ~ooc:(ooc=1) m n
	    and x = randomMatrix ~ooc:(ooc=1) n nrhs in
	    let b = a *$ x in
	    if m >= n then
	      let (q, r) = qr' (copy ~ooc:(ooc=1) a)
	      and bc = copy ~ooc:(ooc=1) b in
	      q @~* bc;
	      subs' r (bc $ (1 <-> n, All));
	      let xh = bc $ (1 <-> n, All) in
	      ok := !ok && (norm1 (a *$ xh -$ b) <= machine_epsilon
			      *. sqrt (float (max m n))
			      *. (norm1 a *. norm1 xh +. norm1 b))
	    (* else
	      let (r, q) = rq' (copy ~ooc:(ooc=1) a)
	      and bc = zeros ~ooc:(ooc=1) n nrhs in
	      (bc $ (1 <-> m, All)) <-$ b;
	      subs' l (bc $ (1 <-> m, All));
	      q @~* bc;
	      ok := !ok && (norm1 (a *$ bc -$ b) <= machine_epsilon
			      *. sqrt (float (max m n))
			      *. (norm1 a *. norm1 bc +. norm1 b))
		 *)
	  done
	done
      done
    done;
    !ok

  let test15 () =
    let ms = [| 20; 21; 22; 23; 24; 25 |]
    and ns = [| 20; 21; 22; 23; 24; 25 |]
    and ok = ref true in
    for m = 0 to Array.length ms - 1 do
      for n = 0 to Array.length ns - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(m)
	  and n = ns.(n) in
	  let a = randomMatrix ~ooc:(ooc=1) m n in
	  let (rF, q) = rq' (copy ~ooc:(ooc=1) a)
	  and r = zeros ~ooc:(ooc=1) m n in
	  if m < n then
	    iDU rF (r $ (All, n-m+1 <-> n))
	  else
	    iDU rF r;
	  r *@ q;
	  ok := !ok && (norm1 (r -$ a) <= sqrt (float (m * n)) *.
			  machine_epsilon *. norm1 a)
	done
      done
    done;
    !ok

  let test16 () =
    let ms = [| 20; 21; 22; 23; 24; 25 |]
    and ns = [| 20; 21; 22; 23; 24; 25 |]
    and ok = ref true in
    for mi = 0 to Array.length ms - 1 do
      for ni = 0 to Array.length ns - 1 do
	for ooc = 0 to 1 do
	  let m = ms.(mi)
	  and n = ns.(ni) in
	  let a = randomMatrix ~ooc:(ooc=1) m n in
	  let (rF, q) = rq' (copy ~ooc:(ooc=1) a)
	  and r = zeros ~ooc:(ooc=1) m n in
	  if m < n then
	    iDU rF (r $ (All, n-m+1 <-> n))
	  else
	    iDU rF r;
	  a *~@ q;
	  ok := !ok && (norm1 (r -$ a) <= sqrt (float (m * n)) *.
			  machine_epsilon *. norm1 a)
	done
      done
    done;
    !ok

  let test17 () =
    let ms = [| 20; 21; 22; 23; 24; 25 |]
    and ns = [| 20; 21; 22; 23; 24; 25 |]
    and nrhss = [| 1; 2; 3; 4 |]
    and ok = ref true in
    for mi = 0 to Array.length ms - 1 do
      for ni = 0 to Array.length ns - 1 do
	for nrhsi = 0 to Array.length nrhss - 1 do
	  for ooc = 0 to 1 do
	    let m = ms.(mi)
	    and n = ns.(ni)
	    and nrhs = nrhss.(nrhsi) in
	    let a = randomMatrix ~ooc:(ooc=1) m n
	    and x = randomMatrix ~ooc:(ooc=1) n nrhs in
	    let b = a *$ x in
	    if m >= n then
	      let (q, r) = qr' (copy ~ooc:(ooc=1) a)
	      and bc = copy ~ooc:(ooc=1) b in
	      q @~* bc;
	      subs' r (bc $ (1 <-> n, All));
	      let xh = bc $ (1 <-> n, All) in
	      ok := !ok && (norm1 (a *$ xh -$ b) <= machine_epsilon
			      *. sqrt (float (max m n))
			      *. (norm1 a *. norm1 xh +. norm1 b))
	    else
	      let (r, q) = rq' (copy ~ooc:(ooc=1) a)
	      and bc = zeros ~ooc:(ooc=1) n nrhs in
	      (bc $ (n - m + 1 <-> n, All)) <-$ b;
	      subs' r (bc $ (n - m + 1 <-> n, All));
	      q @~* bc;
	      ok := !ok && (norm1 (a *$ bc -$ b) <= machine_epsilon
			      *. sqrt (float (max m n))
			      *. (norm1 a *. norm1 bc +. norm1 b))
	  done
	done
      done
    done;
    !ok


  let tests = [| test1; test2; test3; test4; test5; test6; test7; test8;
		 test9; test10; test11; test12; test13; test14; test15;
		 test16; test17 |]

  let run_tests () =
    let suc = ref 0
    and fal = ref 0
    and abo = ref 0
    and start = 1
    and finish = 1000 in
    for i = max 0 (start - 1) to min (Array.length tests - 1) (finish - 1) do
      print_string "Test ";
      print_int (i + 1);
      begin
	try
	  if tests.(i) () then
	    (print_string " : Pass";
	     suc := !suc + 1)
	  else
	    (print_string " : Fail";
	     fal := !fal + 1);
	with
	  Failure s ->
	    print_string " : ";
	    print_string s;
	    abo := !abo + 1
	(*| _ ->
	    print_string " : Abort";
	    abo := !abo + 1*)
      end;
      print_newline ();
    done;
    print_string "Passed: ";
    print_int !suc;
    print_string "\nFailed: ";
    print_int !fal;
    print_string "\nAborted: ";
    print_int !abo;
    print_newline ()
      
end;; (* module Tester *)


module Dtests = Tester (DBlop);;
module Stests = Tester (SBlop);;
module Ztests = Tester (ZBlop);;
module Ctests = Tester (CBlop);;

let main () =
  begin
    print_string "Double Precision Tests:\n";
    Dtests.run_tests ();
    print_newline ();

    print_string "Single Precision Tests:\n";
    Stests.run_tests ();
    print_newline ();
    
    print_string "Single Complex Tests:\n";
    Ctests.run_tests ();
    print_newline ();

    print_string "Double Complex Tests:\n";
    Ztests.run_tests ();
    print_newline ();

  end;;

main ();;
