(*
  File: timeNla.ml
  Author: Shivkumar Chandrasekaran
  Email: shiv@ece.ucsb.edu
  Date: 4.26.2004
  Package: CamlFloat
  Purpose: Timings for basic stuff
*)

open Bigarray;;
open Nla;;


module Timer (Blop : BLOP) =
  struct

    open Blop

    let time1 () =
      let ns = [| 256; 512; 1024; 2048; 4096; 4096*2; 4096*4; 4096*8 |] in
      for ni = 0 to Array.length ns - 1 do
	let n = ns.(ni) in
	let a = randomMatrix n n
	and b = randomMatrix n 1 in
	let t1 = Sys.time () in
	a |/| b;
	let t2 = Sys.time () in
	Printf.printf "n=%i time=%g" n (t2 -. t1);
	print_newline ()
      done


    let timers = [| time1 |]
	
    let run_timers () =
      let first_timer = 1
      and last_timer = 200 in
      (* Random.self_init (); *)
      for i = max 0 (first_timer - 1) to
	min (Array.length timers) last_timer - 1 do
	timers.(i) ()
      done;
      print_newline ()
	
  end;; (* module Timer *)


module Dtests = Timer (DBlop);;
module Stests = Timer (SBlop);;
module Ztests = Timer (ZBlop);;
module Ctests = Timer (CBlop);;

let main () =
  print_string "Double Precision Times:\n";
  Dtests.run_timers ();
  print_newline ();

  print_string "Single Precision Times:\n";
  Stests.run_timers ();
  print_newline ();

  print_string "Double Complex Times:\n";
  Ztests.run_timers ();
  print_newline ();

  print_string "Single Complex Times:\n";
  Ctests.run_timers ();;

main ();;
