(* File : myocamlbuild.ml
   Author : Shivkumar Chandrasekaran
   Date : 1.30.2008
   Purpose : Required by ocamlbuild

   $Id$
*)

(* Inferred from the example in
   http://brion.inria.fr/gallium/index.php/Ocamlbuild_example_with_C_stubs
*)

open Ocamlbuild_plugin
open Command

(* Configuration section *)
(* This is for Linux x64 at the moment *)
let lapack_dir  = "-L/usr/lib64/atlas"
let cblas_lib   = "-lcblas"
let f77blas_lib = "-lf77blas"
let atlas_lib   = "-latlas"
let g2c_lib     = "-lg2c"
let lapack_lib  = [A lapack_dir; A cblas_lib; A f77blas_lib; A atlas_lib;
		   A g2c_lib]
let lapack_lib_camloptions =
  [A "-ccopt"; A lapack_dir;
   A "-cclib"; A cblas_lib;
   A "-cclib"; A f77blas_lib;
   A "-cclib"; A atlas_lib;
   A "-cclib"; A g2c_lib]

(* Force -custom in byte code *)
let static = true

(* List of headers *)
let headers = ["cblas.h"; "blop_c.h"]


dispatch begin function
  | After_rules ->

      (* When one makes a C library that uses lapack with ocamlmklib,
	 then issue these flags. *)
      flag ["ocamlmklib"; "c"; "use_lapack"] (S lapack_lib);

      (* When one compiles C code using lapack -- no include directories *)
      (* flag ["c"; "compile"; "include_lapack"] *)

      flag ["link"; "ocaml"; "library"; "use_lapack"]
	(S lapack_lib_camloptions);

      (* If `static' is true then every ocaml link in bytecode will add
	 -custom *)
      if static then flag ["link"; "ocaml"; "byte"] (A "-custom");

end
